package admin

import (
	"context"
	"gd_admin/apis"
	"gd_admin/common.in/config"
	"gd_admin/common.in/jsonrpc2"
	"gd_admin/common.in/utils"
	"gd_admin/errors"
	"gd_admin/impl/dbmodel"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/astaxie/beego/orm"
	"go.uber.org/zap"
	"strings"
)

const (
	Page_Size = 10
)

func getSql(req *apis.AdminGetAllReq) string {
	sql := "SELECT a.*,c.`name` as group_name FROM t_gd_admin a LEFT JOIN t_gd_admin_rbac_access b ON a.id=b.uid LEFT JOIN t_gd_admin_rbac_group c ON b.group_id=c.id"

	if req.Search != "" {
		sql += " WHERE a.admin_name LIKE \"%" + req.Search + "%\""
	}

	sql += " Order By a.id DESC"

	return sql
}

func AdminGetAll(ctx context.Context, req *apis.AdminGetAllReq, reply *apis.AdminGetAllReply) error {
	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err := fmt.Errorf("%+v", r)
			e := &jsonrpc2.Error{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				l.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	// 获取sql
	sql := getSql(req)

	var result []apis.GetAdminList

	rows, err := orm.NewOrm().Raw(sql).QueryRows(&result)
	if err != nil {
		if err == orm.ErrNoRows {
			return errors.AdminsNotExist
		}
		l.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", utils.MarshalJsonString(req)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}

	reply.Total = rows
	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	if req.PageSize == 0 {
		req.PageSize = Page_Size
	}
	start := (req.PageNumber - 1) * req.PageSize
	end := req.PageNumber * req.PageSize

	if end > rows {
		reply.Admins = append(reply.Admins, result[start:]...)
	} else {
		reply.Admins = append(reply.Admins, result[start:end]...)
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize

	return nil
}

// 获取用户详情
func GetInfo(ctx context.Context, req *apis.GetInfoReq, reply *apis.GetInfoReply) error {
	if req.Uid <= 0 {
		return errors.ArgsError
	}

	p := apis.AdminInfo{}
	sql := `SELECT admin_name, admin_password, admin_phone, admin_email FROM t_gd_admin WHERE id = ?`

	err := orm.NewOrm().Raw(sql, req.Uid).QueryRow(&p)
	if err != nil {
		if err == orm.ErrNoRows {
			return errors.AdminNotExist
		}
		l.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", utils.MarshalJsonString(req)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}
	bytes, _ := base64.StdEncoding.DecodeString(p.AdminPassword)
	cryPasswd, _ := config.AesDecrypt(bytes, []byte(CRYPTO_KEY))
	fmt.Printf("%s", cryPasswd)
	reply.AdminName = p.AdminName
	reply.AdminEmail = p.AdminEmail
	reply.AdminPassword = string(cryPasswd)
	reply.AdminPhone = p.AdminPhone

	// 获取分组信息
	//  where
	filter := map[string]interface{}{
		"uid": req.Uid,
	}

	n := dbmodel.TGdAdminRbacAccess{}
	err = n.Fetch(orm.NewOrm(), filter)
	if err != nil {
		l.Error("mysql",
			zap.String("sql", fmt.Sprintf("SELECT * FROM %s", n.TableName())),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}

	// 获取节点详情
	//  where
	where := map[string]interface{}{
		"id": n.GroupId,
	}
	g := dbmodel.TGdAdminRbacGroup{}
	err = g.Fetch(orm.NewOrm(), where)
	if err != nil {
		l.Error("mysql",
			zap.String("sql", fmt.Sprintf("SELECT * FROM %s", n.TableName())),
			zap.String("fields", utils.MarshalJsonString(where)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}

	reply.RbcInfo.Name = g.Name
	reply.RbcInfo.GroupId = g.Id
	reply.RbcInfo.NodeId = strings.Split(g.RbacNodeList, ",")

	return nil
}
