package admin

import (
	"context"
	"gd_admin/apis"
	"gd_admin/common.in/config"
	"gd_admin/common.in/jsonrpc2"
	"gd_admin/common.in/storage"
	"gd_admin/common.in/utils"
	"gd_admin/errors"
	"gd_admin/impl/rbac"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/astaxie/beego/orm"
	"go.uber.org/zap"
	"time"
)

func AdminRegister(ctx context.Context, req *apis.AdminRegisterReq, reply *apis.AdminRegisterReply) (err error) {
	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &jsonrpc2.Error{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				l.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.AdminName == "" || req.AdminPasswd == "" || req.NodeId == "" || req.GroupId <= 0 {
		return errors.ArgsError
	}

	var uid int64

	insertAdminTask := func(db orm.Ormer) error {
		adminInfo := apis.AdminInfo{}
		cryPasswd, err := config.AesEncrypt(req.AdminPasswd, CRYPTO_KEY)
		if err != nil {
			return errors.ArgsError
		}
		adminInfo.AdminPassword = base64.StdEncoding.EncodeToString(cryPasswd)

		if req.AdminPhone != "" {
			adminInfo.AdminPhone = req.AdminPhone
		}

		if req.AdminEmail != "" {
			adminInfo.AdminEmail = req.AdminEmail
		}

		o := orm.NewOrm()
		var adminId int64

		err = o.Raw("select id from t_gd_admin where admin_name = ? and status = 0", req.AdminName).QueryRow(&adminId)
		if err != nil && err != orm.ErrNoRows {
			l.Error("mysql",
				zap.String("sql", "select * from t_gd_admin"),
				zap.String("fields", utils.MarshalJsonString(req.AdminName)),
				zap.String("error", err.Error()))
			return errors.DataBaseError
		}

		if adminId > 0 {
			return errors.AdminExist
		}

		adminInfo.AdminName = req.AdminName

		if req.AdminPhone != "" {
			adminInfo.AdminPhone = req.AdminPhone
		}

		adminInfo.Status = 0
		adminInfo.CreatedAt = time.Now().Format("2006-01-02 15:04:05")
		adminInfo.UpdatedAt = time.Now().Format("2006-01-02 15:04:05")

		uid, err = o.Insert(&adminInfo)
		if err != nil {
			l.Error("mysql",
				zap.String("sql", "Insert from t_gd_admin"),
				zap.String("fields", utils.MarshalJsonString(adminInfo)),
				zap.String("error", err.Error()))
			return errors.DataBaseError
		}

		reply.Id = uid

		return nil
	}

	updateGroupInfo := func(db orm.Ormer) error {
		// 查看是否需要新增分组
		//groupId, err := rbac.AddCostomGroup(db, req.NodeId)
		//if err != nil {
		//	return err
		//}

		// 新增访问权限分组
		err := rbac.AddAccess(db, uid, int64(req.GroupId))
		if err != nil {
			return err
		}

		return nil
	}

	//处理事务
	tasks := []storage.DbaTasker{}
	tasks = append(tasks, storage.GenerateDbaTask(insertAdminTask))
	tasks = append(tasks, storage.GenerateDbaTask(updateGroupInfo))
	storage.ExecTrans(tasks...)

	return nil
}
